@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2014
+  Copyright (c) 1998 - 2015
   ILK  -  Tilburg University
   CNTS -  University of Antwerp
 
@@ -30,7 +30,7 @@
 #include <signal.h>
 #include <iostream>
 #include "ticcutils/StringOps.h"
-#include "timblserver/SocketBasics.h"
+#include "ticcutils/SocketBasics.h"
 
 using namespace std;
 
