@@ -1,8 +1,6 @@
 /*
-  $Id: ClientBase.cxx 16885 2014-01-28 14:27:12Z sloot $
-  $URL: https://ilk.uvt.nl/svn/trunk/sources/TimblServer/src/ClientBase.cxx $
-
-  Copyright (c) 1998 - 2014
+  Copyright (c) 1998 - 2016
+  CLST  - Radboud University
   ILK   - Tilburg University
   CLiPS - University of Antwerp
 
@@ -22,9 +20,10 @@
   along with this program; if not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/timblserver/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
 #include <string>
@@ -32,7 +31,6 @@
 #include <cstdlib>
 #include <csignal>
 #include "ticcutils/StringOps.h"
-#include "timblserver/FdStream.h"
 #include "timblserver/ClientBase.h"
 
 using namespace std;
@@ -290,12 +288,12 @@ namespace TimblServer {
     if ( client.isValid() ) {
       string request;
       while( getline( is, request ) ){
-	//      cerr << "script line " << request << endl;
+	//	cerr << "script line " << request << endl;
 	if ( client.write( request + "\n" ) ){
 	  string response;
 	  bool more = true;
 	  while ( more && client.read( response ) ){
-	    //	  cerr << "response line " << response << endl;
+	    //	    cerr << "response line " << response << endl;
 	    if ( response.empty() ){
 	      continue;
 	    }
