@@ -1,11 +1,9 @@
 /*
-  $Id: TimblClient.cxx 16821 2014-01-06 10:16:43Z sloot $
-  $URL: https://ilk.uvt.nl/svn/trunk/sources/TimblServer/src/TimblClient.cxx $
-
-  Copyright (c) 1998 - 2014
+  Copyright (c) 1998 - 2016
+  CLST  - Radboud University
   ILK   - Tilburg University
   CLiPS - University of Antwerp
- 
+
   This file is part of timblserver
 
   timblserver is free software; you can redistribute it and/or modify
@@ -22,9 +20,10 @@
   along with this program; if not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/timblserver/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
 #include <string>
@@ -40,7 +39,8 @@
 using namespace std;
 
 #include "timbl/TimblAPI.h"
-#include "timblserver/SocketBasics.h"
+#include "ticcutils/CommandLine.h"
+#include "ticcutils/SocketBasics.h"
 #include "timblserver/ClientBase.h"
 
 using namespace std;
@@ -66,42 +66,49 @@ int main(int argc, char *argv[] ){
   string base;
   string node;
   string port;
-  TimblOpts opts( argc, argv );
+  TiCC::CL_Options opts( "i:o:p:n:b:", "batch" );
+  try {
+    opts.init( argc, argv );
+  }
+  catch( TiCC::OptionError& e ){
+    cerr << e.what() << endl;
+    usage();
+    exit(EXIT_FAILURE);
+  }
   string value;
-
-  if ( opts.Find( "i", value ) ){
-    if ( (input_file.open( value.c_str(), ios::in ), !input_file.good() ) ){
+  if ( opts.extract( "i", value ) ){
+    if ( (input_file.open( value, ios::in ), !input_file.good() ) ){
       cerr << argv[0] << " - couldn't open inputfile " << value << endl;
       exit(EXIT_FAILURE);
     }
     cout << "reading input from: " << value << endl;
     Input = &input_file;
   }
-  if ( opts.Find( "o", value ) ){
-    if ( (output_file.open( value.c_str(), ios::out ), !output_file.good() ) ){
+  if ( opts.extract( "o", value ) ){
+    if ( (output_file.open( value, ios::out ), !output_file.good() ) ){
       cerr << argv[0] << " - couldn't open outputfile " << value << endl;
       exit(EXIT_FAILURE);
     }
     cout << "writing output to: " << value << endl;
     Output = &output_file;
   }
-  if ( opts.Find( "batch", value ) ){
+  if ( opts.extract( "batch" ) ){
     c_mode = true;
   }
-  if ( opts.Find( "p", value ) ){
+  if ( opts.extract( "p", value ) ){
     port = value;
   }
-  if ( opts.Find( "n", value ) ){
+  if ( opts.extract( "n", value ) ){
     node = value;
   }
-  if ( opts.Find( "b", value ) ){
+  if ( opts.extract( "b", value ) ){
     base = value;
   }
   if ( !node.empty() && !port.empty() ){
     TimblServer::ClientClass client;
     if ( !client.connect( node, port ) ){
       cerr << "connection failed " << endl;
-      exit(EXIT_FAILURE);      
+      exit(EXIT_FAILURE);
     }
     if ( !base.empty() ){
       if ( !client.setBase( base ) ){
