@@ -1,8 +1,6 @@
 /*
-  $Id: TimblServer.cxx 17369 2014-06-23 08:24:48Z sloot $
-  $URL: https://ilk.uvt.nl/svn/trunk/sources/TimblServer/src/TimblServer.cxx $
-
-  Copyright (c) 1998 - 2014
+  Copyright (c) 1998 - 2016
+  CLST  - Radboud University
   ILK   - Tilburg University
   CLiPS - University of Antwerp
 
@@ -22,27 +20,29 @@
   along with this program; if not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/timblserver/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
 #include <exception>
 #include <vector>
 #include <string>
+#include <cstdlib>
 
+#include "ticcutils/CommandLine.h"
+#include "ticcutils/Timer.h"
 #include "timbl/TimblAPI.h"
 #include "timbl/GetOptClass.h"
-#include "timblserver/FdStream.h"
-#include "timblserver/ServerBase.h"
+#include "ticcutils/ServerBase.h"
 
 using namespace std;
 using namespace Timbl;
 using namespace TimblServer;
-using namespace TiCC;
 
-bool Do_Server = false;
-bool Do_Multi_Server = false;
+#define DBG *TiCC::Dbg(myLog)
+#define LOG *TiCC::Log(myLog)
 
 inline void usage_full(void){
   cerr << "usage: timblserver [TiMBLoptions] [ServerOptions]" << endl << endl;
@@ -67,51 +67,15 @@ inline void usage(void){
   cerr << endl;
 }
 
-class softExit : public exception {};
 
-void Preset_Values( TimblOpts& Opts ){
-  bool mood;
-  string value;
-  if ( Opts.Find( 'h', value, mood ) ){
-    usage_full();
-    throw( softExit() );
-  }
-  if ( Opts.Find( 'V', value, mood ) ){
-    cerr << "TiMBL server " << ServerBase::VersionInfo( true ) << endl;
-    cerr << "Based on TiMBL " << TimblAPI::VersionInfo( true ) << endl;
-    throw( softExit() );
-  }
-  Opts.Add( 'v', "F", true );
-  Opts.Add( 'v', "S", false );
-  if ( Opts.Find( "config", value, mood ) ){
-    Do_Multi_Server = true;
-  }
-  if ( Opts.Find( 'S', value, mood ) ){
-    if ( Do_Multi_Server ){
-      cerr << "options -S conflicts with option --config" << endl;
-      throw( softExit() );
-    }
-    else {
-      Do_Server = true;
-    }
-  }
-  else {
-    if ( Do_Multi_Server ){
-      Opts.Add( 'S', "0", true );
-      // hack to signal GetOptClass that we are going into server mode
-    }
-  }
-}
-
-void startExperiments( ServerBase *server,
-		       TimblOpts& opts ){
+void startExperiments( ServerBase *server ){
   map<string,string> allvals;
   if ( server->config->hasSection("experiments") )
     allvals = server->config->lookUpAll("experiments");
   else {
     allvals = server->config->lookUpAll("global");
     // old style, everything is global
-    // remove all alreday processed stuff
+    // remove all already processed stuff
     map<string,string>::iterator it = allvals.begin();
     while ( it != allvals.end() ){
       if ( it->first == "port" ||
@@ -120,6 +84,7 @@ void startExperiments( ServerBase *server,
 	   it->first == "debug" ||
 	   it->first == "pidfile" ||
 	   it->first == "daemonize" ||
+	   it->first == "configDir" ||
 	   it->first == "maxconn" ){
 	allvals.erase(it++);
       }
@@ -128,21 +93,71 @@ void startExperiments( ServerBase *server,
       }
     }
   }
-  map<string, TimblExperiment*> *experiments = new map<string, TimblExperiment*>();
+  if ( allvals.empty() ){
+    cerr << "Unable to initalize at least one tagger" << endl
+	 << "please check your commandline and/or your confiuration file"
+	 << endl;
+    exit(EXIT_FAILURE);
+  }
+
+  auto experiments = new map<string, TimblExperiment*>();
   server->callback_data = experiments;
 
-  if ( allvals.empty() ){
-    cerr << "opts" << opts << endl;
-    // old style stuff
+  map<string,string>::iterator it = allvals.begin();
+  while ( it != allvals.end() ){
+    TiCC::CL_Options opts;
+    opts.set_short_options( timbl_short_opts + serv_short_opts );
+    opts.set_long_options( timbl_long_opts + serv_long_opts );
+    opts.init( it->second );
     string treeName;
     string trainName;
-    bool mood;
-    if ( opts.Find( 'f', trainName, mood ) )
-      opts.Delete( 'f' );
-    else if ( opts.Find( 'i', treeName, mood ) )
-      opts.Delete( 'i' );
+    string MatrixInFile = "";
+    string WgtInFile = "";
+    Weighting WgtType = GR;
+    Algorithm algorithm = IB1;
+    string ProbInFile = "";
+    string value;
+    if ( opts.is_present( 'a', value ) ){
+      // the user gave an algorithm
+      if ( !string_to( value, algorithm ) ){
+	cerr << "illegal -a value: " << value << endl;
+	exit(EXIT_FAILURE);
+      }
+    }
+    if ( !opts.extract( 'f', trainName ) )
+      opts.extract( 'i', treeName );
+    if ( opts.extract( 'u', ProbInFile ) ){
+      if ( algorithm == IGTREE ){
+	cerr << "-u option is useless for IGtree" << endl;
+	exit(EXIT_FAILURE);
+      }
+    }
+    if ( opts.extract( 'w', value ) ){
+      Weighting W;
+      if ( !string_to( value, W ) ){
+	// No valid weighting, so assume it also has a filename
+	vector<string> parts;
+	size_t num = TiCC::split_at( value, parts, ":" );
+	if ( num == 2 ){
+	  if ( !string_to( parts[1], W ) ){
+	    cerr << "invalid weighting option: " << value << endl;
+	    exit(1);
+	  }
+	  WgtInFile = parts[0];
+	  WgtType = W;
+	}
+	else if ( num == 1 ){
+	  WgtInFile = value;
+	}
+	else {
+	  cerr << "invalid weighting option: " << value << endl;
+	  exit(1);
+	}
+      }
+    }
+    opts.extract( "matrixin", MatrixInFile );
     if ( !( treeName.empty() && trainName.empty() ) ){
-      TimblAPI *run = new TimblAPI( &opts );
+      TimblAPI *run = new TimblAPI( opts, it->first );
       bool result = false;
       if ( run && run->Valid() ){
 	if ( treeName.empty() ){
@@ -153,161 +168,32 @@ void startExperiments( ServerBase *server,
 	  server->myLog << "treeName = " << treeName << endl;
 	  result = run->GetInstanceBase( treeName );
 	}
+	if ( result && WgtInFile != "" ) {
+	  result = run->GetWeights( WgtInFile, WgtType );
+	}
+	if ( result && ProbInFile != "" )
+	  result = run->GetArrays( ProbInFile );
+	if ( result && MatrixInFile != "" ) {
+	  result = run->GetMatrices( MatrixInFile );
+	}
       }
       if ( result ){
 	run->initExperiment();
-	(*experiments)["default"] = run->grabAndDisconnectExp();
-	server->myLog << "started classic experiment " << endl;
+	(*experiments)[it->first] = run->grabAndDisconnectExp();
+	delete run;
+	server->myLog << "started experiment " << it->first
+		      << " with parameters: " << it->second << endl;
       }
       else {
-	server->myLog << "FAILED to start classic experiment " << endl;
+	server->myLog << "FAILED to start experiment " << it->first
+		      << " with parameters: " << it->second << endl;
       }
     }
     else {
-      server->myLog << "missing '-i' or '-f' option on command line" << endl;
+      server->myLog << "missing '-i' or '-f' option in serverconfig entry: '"
+		    << it->first << "=" << it->second << "'" << endl;
     }
-  }
-  else {
-    map<string,string>::iterator it = allvals.begin();
-    while ( it != allvals.end() ){
-      cerr << "OPTS: " << it->second << endl;
-      TimblOpts opts( it->second );
-      string treeName;
-      string trainName;
-      bool mood;
-      if ( opts.Find( 'f', trainName, mood ) )
-	opts.Delete( 'f' );
-      else if ( opts.Find( 'i', treeName, mood ) )
-	opts.Delete( 'i' );
-      if ( !( treeName.empty() && trainName.empty() ) ){
-	TimblAPI *run = new TimblAPI( &opts, it->first );
-	bool result = false;
-	if ( run && run->Valid() ){
-	  if ( treeName.empty() ){
-	    server->myLog << "trainName = " << trainName << endl;
-	    result = run->Learn( trainName );
-	  }
-	  else {
-	    server->myLog << "treeName = " << treeName << endl;
-	    result = run->GetInstanceBase( treeName );
-	  }
-	}
-	if ( result ){
-	  run->initExperiment();
-	  (*experiments)[it->first] = run->grabAndDisconnectExp();
-	  delete run;
-	  server->myLog << "started experiment " << it->first
-			<< " with parameters: " << it->second << endl;
-	}
-	else {
-	  server->myLog << "FAILED to start experiment " << it->first
-			<< " with parameters: " << it->second << endl;
-	}
-      }
-      else {
-	server->myLog << "missing '-i' or '-f' option in serverconfig file" << endl;
-      }
-      ++it;
-    }
-  }
-}
-
-void startClassicExperiment( ServerBase *server,
-			     TimblOpts& opts ){
-  string treeName;
-  string trainName;
-  string MatrixInFile = "";
-  string WgtInFile = "";
-  Weighting WgtType = GR;
-  Algorithm algorithm = IB1;
-  string ProbInFile = "";
-  string value;
-  bool mood;
-  if ( opts.Find( 'a', value, mood ) ){
-    // the user gave an algorithm
-    if ( !string_to( value, algorithm ) ){
-      cerr << "illegal -a value: " << value << endl;
-      exit(1);
-    }
-  }
-  if ( opts.Find( 'f', trainName, mood ) )
-    opts.Delete( 'f' );
-  if ( opts.Find( 'P', value, mood ) ){
-    cerr << "illegal option, value = " << value << endl;
-    exit(1);
-  }
-  if ( opts.Find( "matrixin", MatrixInFile, mood ) ){
-    opts.Delete( "matrixin" );
-  }
-  if ( opts.Find( 'i', treeName, mood ) ){
-    opts.Delete( 'i' );
-  }
-  if ( opts.Find( 'u', ProbInFile, mood ) ){
-    if ( algorithm == IGTREE ){
-      cerr << "-u option is useless for IGtree" << endl;
-      exit(1);
-    }
-    opts.Delete( 'u' );
-  }
-  if ( opts.Find( 'w', value, mood ) ){
-    Weighting W;
-    if ( !string_to( value, W ) ){
-      // No valid weighting, so assume it also has a filename
-      vector<string> parts;
-      size_t num = TiCC::split_at( value, parts, ":" );
-      if ( num == 2 ){
-	if ( !string_to( parts[1], W ) ){
-	  cerr << "invalid weighting option: " << value << endl;
-	  exit(1);
-	}
-	WgtInFile = parts[0];
-	WgtType = W;
-      }
-      else if ( num == 1 ){
-	WgtInFile = value;
-      }
-      else {
-	cerr << "invalid weighting option: " << value << endl;
-	exit(1);
-      }
-      opts.Delete( 'w' );
-    }
-  }
-  if ( !( treeName.empty() && trainName.empty() ) ){
-    TimblAPI *run = new TimblAPI( &opts, "default" );
-    bool result = false;
-    if ( run && run->Valid() ){
-      if ( treeName.empty() ){
-	cerr << "trainName = " << trainName << endl;
-	result = run->Learn( trainName );
-      }
-      else {
-	cerr << "treeName = " << treeName << endl;
-	result = run->GetInstanceBase( treeName );
-      }
-      if ( result && WgtInFile != "" ) {
-	result = run->GetWeights( WgtInFile, WgtType );
-      }
-      if ( result && ProbInFile != "" )
-	result = run->GetArrays( ProbInFile );
-      if ( result && MatrixInFile != "" ) {
-	result = run->GetMatrices( MatrixInFile );
-      }
-    }
-    if ( result ){
-      run->initExperiment();
-      map<string, TimblExperiment*> *experiments = new map<string, TimblExperiment*>();
-      server->callback_data = experiments;
-      (*experiments)["default"] = run->grabAndDisconnectExp();
-      delete run;
-      cerr << "started classic experiment " << endl;
-    }
-    else {
-      cerr << "FAILED to start experiment " << endl;
-    }
-  }
-  else {
-    cerr << "missing '-i' or '-f' option on the command line " << endl;
+    ++it;
   }
 }
 
@@ -334,7 +220,7 @@ TimblExperiment *createClient( const TimblExperiment *exp,
   if ( exp->getOptParams() ){
     result->setOptParams( exp->getOptParams()->Clone( &(args->os() ) ) );
   }
-  result->setExpName(string("exp-")+toString( args->id() ) );
+  result->setExpName(string("exp-")+TiCC::toString( args->id() ) );
   return result;
 }
 
@@ -378,7 +264,7 @@ public:
   void showSettings(){ _exp->ShowSettings( os ); };
   bool setOptions( const string& param );
 private:
-  LogStream& myLog;
+  TiCC::LogStream& myLog;
   bool doDebug;
   TimblExperiment *_exp;
   ostream& os;
@@ -392,6 +278,8 @@ TimblClient::TimblClient( TimblExperiment *exp,
   os(args->os()),
   is(args->is())
 {
+  if ( doDebug )
+    myLog.setlevel(LogHeavy);
   _exp = exp->clone();
   *_exp = *exp;
   if ( !_exp->connectToSocket( &(args->os() ) ) ){
@@ -400,21 +288,20 @@ TimblClient::TimblClient( TimblExperiment *exp,
   if ( exp->getOptParams() ){
     _exp->setOptParams( exp->getOptParams()->Clone( &(args->os() ) ) );
   }
-  _exp->setExpName(string("exp-")+toString( args->id() ) );
+  _exp->setExpName(string("exp-")+TiCC::toString( args->id() ) );
 }
 
 bool TimblClient::setOptions( const string& param ){
   if ( _exp->SetOptions( param ) ){
-    if ( doDebug )
-      *Log(myLog) << "setOptions: " << param << endl;
+    DBG << "setOptions: " << param << endl;
     if ( _exp->ConfirmOptions() )
       os << "OK" << endl;
     else
       os << "ERROR { set options failed: " << param << "}" << endl;
   }
   else {
-    if ( doDebug )
-      *Log(myLog) << ": Don't understand '" << param << "'" << endl;
+    DBG << ": Don't understand '" << param << "'" << endl;
+    os << "ERROR { set options failed: " << param << "}" << endl;
   }
   return true;
 }
@@ -424,10 +311,9 @@ bool TimblClient::classifyLine( const string& params ){
   string Distrib;
   string Answer;
   if ( _exp->Classify( params, Answer, Distrib, Distance ) ){
-    if ( doDebug )
-      *Log(myLog) << _exp->ExpName() << ":" << params << " --> "
-		  << Answer << " " << Distrib
-		  << " " << Distance << endl;
+    DBG << _exp->ExpName() << ":" << params << " --> "
+		<< Answer << " " << Distrib
+		<< " " << Distance << endl;
     os << "CATEGORY {" << Answer << "}";
     if ( os.good() ){
       if ( _exp->Verbosity(DISTRIB) ){
@@ -456,9 +342,8 @@ bool TimblClient::classifyLine( const string& params ){
     return os.good();
   }
   else {
-    if ( doDebug )
-      *Log(myLog) << _exp->ExpName() << ": Classify Failed on '"
-		  << params << "'" << endl;
+    DBG << _exp->ExpName() << ": Classify Failed on '"
+		<< params << "'" << endl;
     return false;
   }
 }
@@ -477,15 +362,11 @@ void TcpServer::callback( childArgs *args ){
   if ( experiments->size() == 1
        && experiments->find("default") != experiments->end() ){
     baseName = "default";
-    *Dbg(myLog) << " Voor Create Default Client " << endl;
+    DBG << " Voor Create Default Client " << endl;
     client = new TimblClient( (*experiments)[baseName], args );
-    *Dbg(myLog) << " Na Create Client " << endl;
+    DBG << " Na Create Client " << endl;
     // report connection to the server terminal
     //
-    char line[256];
-    sprintf( line, "Thread %zd, on Socket %d", (uintptr_t)pthread_self(),
-	     sockId );
-    *Log(myLog) << line << ", started." << endl;
   }
   else {
     args->os() << "available bases: ";
@@ -497,17 +378,17 @@ void TcpServer::callback( childArgs *args ){
     args->os() << endl;
   }
   if ( getline( args->is(), Line ) ){
-    *Dbg(myLog) << "FirstLine='" << Line << "'" << endl;
+    DBG << "FirstLine='" << Line << "'" << endl;
     string Command, Param;
     bool go_on = true;
-    *Dbg(myLog) << "running FromSocket: " << sockId << endl;
+    DBG << "running FromSocket: " << sockId << endl;
 
     do {
-      Line = trim( Line );
-      *Dbg(myLog) << "Line='" << Line << "'" << endl;
+      Line = TiCC::trim( Line );
+      DBG << "Line='" << Line << "'" << endl;
       Split( Line, Command, Param );
-      *Dbg(myLog) << "Command='" << Command << "'" << endl;
-      *Dbg(myLog) << "Param='" << Param << "'" << endl;
+      DBG << "Command='" << Command << "'" << endl;
+      DBG << "Param='" << Param << "'" << endl;
       switch ( check_command(Command) ){
       case Base:{
 	map<string,TimblExperiment*>::const_iterator it
@@ -517,15 +398,15 @@ void TcpServer::callback( childArgs *args ){
 	  args->os() << "selected base: '" << Param << "'" << endl;
 	  if ( client )
 	    delete client;
-	  *Dbg(myLog) << " Voor Create Default Client " << endl;
+	  DBG << " Voor Create Default Client " << endl;
 	  client = new TimblClient( it->second, args );
-	  *Dbg(myLog) << " Na Create Client " << endl;
+	  DBG << " Na Create Client " << endl;
 	  // report connection to the server terminal
 	  //
 	  char line[256];
 	  sprintf( line, "Thread %zd, on Socket %d",
 		   (uintptr_t)pthread_self(), sockId );
-	  *Log(myLog) << line << ", started." << endl;
+	  LOG << line << ", started." << endl;
 	}
 	else {
 	  args->os() << "ERROR { Unknown basename: " << Param << "}" << endl;
@@ -567,9 +448,8 @@ void TcpServer::callback( childArgs *args ){
 	args->os() << "SKIP '" << Line << "'" << endl;
 	break;
       default:
-	if ( doDebug() )
-	  *Log(myLog) << sockId << ": Don't understand '"
-		      << Line << "'" << endl;
+	DBG << sockId << ": Don't understand '"
+		    << Line << "'" << endl;
 	args->os() << "ERROR { Illegal instruction:'" << Command
 		   << "' in line:" << Line << "}" << endl;
 	break;
@@ -578,7 +458,7 @@ void TcpServer::callback( childArgs *args ){
     while ( go_on && getline( args->is(), Line ) );
   }
   delete client;
-  *Log(myLog) << "Thread " << (uintptr_t)pthread_self()
+  LOG << "Thread " << (uintptr_t)pthread_self()
 	      << " terminated, " << result
 	      << " instances processed " << endl;
 }
@@ -626,11 +506,11 @@ void HttpServer::callback( childArgs *args ){
   char logLine[256];
   sprintf( logLine, "Thread %zd, on Socket %d", (uintptr_t)pthread_self(),
 	   args->id() );
-  *Log(myLog) << logLine << ", started." << endl;
+  LOG << logLine << ", started." << endl;
   string Line;
   int timeout = 1;
   if ( nb_getline( args->is(), Line, timeout ) ){
-    *Dbg(myLog) << "FirstLine='" << Line << "'" << endl;
+    DBG << "FirstLine='" << Line << "'" << endl;
     if ( Line.find( "HTTP" ) != string::npos ){
       // skip HTTP header
       string tmp;
@@ -642,7 +522,7 @@ void HttpServer::callback( childArgs *args ){
       if ( spos != string::npos ){
 	string::size_type epos = Line.find( " HTTP" );
 	string line = Line.substr( spos+3, epos - spos - 3 );
-	*Dbg(myLog) << "Line='" << line << "'" << endl;
+	DBG << "Line='" << line << "'" << endl;
 	epos = line.find( "?" );
 	string basename;
 	if ( epos != string::npos ){
@@ -655,22 +535,22 @@ void HttpServer::callback( childArgs *args ){
 	    if ( it != experiments->end() ){
 	      TimblExperiment *api = createClient( it->second, args );
 	      if ( api ){
-		LogStream LS( &myLog );
-		LogStream DS( &myLog );
+		TiCC::LogStream LS( &myLog );
+		TiCC::LogStream DS( &myLog );
 		DS.message(logLine);
 		LS.message(logLine);
 		DS.setstamp( StampBoth );
 		LS.setstamp( StampBoth );
-		XmlDoc doc( "TiMblResult" );
+		TiCC::XmlDoc doc( "TiMblResult" );
 		xmlNode *root = doc.getRoot();
-		XmlSetAttribute( root, "algorithm", toString(api->Algorithm()) );
+		TiCC::XmlSetAttribute( root, "algorithm", TiCC::toString(api->Algorithm()) );
 		vector<string> avs;
-		int avNum = split_at( qstring, avs, "&" );
+		int avNum = TiCC::split_at( qstring, avs, "&" );
 		if ( avNum > 0 ){
 		  multimap<string,string> acts;
 		  for ( int i=0; i < avNum; ++i ){
 		    vector<string> parts;
-		    int num = split_at( avs[i], parts, "=" );
+		    int num = TiCC::split_at( avs[i], parts, "=" );
 		    if ( num == 2 ){
 		      acts.insert( make_pair(parts[0], parts[1]) );
 		    }
@@ -758,19 +638,19 @@ void HttpServer::callback( childArgs *args ){
 			   << ", distance " << distance
 			   << endl;
 
-		      xmlNode *cl = XmlNewChild( root, "classification" );
-		      XmlNewTextChild( cl, "input", params );
-		      XmlNewTextChild( cl, "category", answer );
+		      xmlNode *cl = TiCC::XmlNewChild( root, "classification" );
+		      TiCC::XmlNewTextChild( cl, "input", params );
+		      TiCC::XmlNewTextChild( cl, "category", answer );
 		      if ( api->Verbosity(DISTRIB) ){
-			XmlNewTextChild( cl, "distribution", distrib );
+			TiCC::XmlNewTextChild( cl, "distribution", distrib );
 		      }
 		      if ( api->Verbosity(DISTANCE) ){
-			XmlNewTextChild( cl, "distance",
-					 toString<double>(distance) );
+			TiCC::XmlNewTextChild( cl, "distance",
+					       TiCC::toString<double>(distance) );
 		      }
 		      if ( api->Verbosity(MATCH_DEPTH) ){
-			XmlNewTextChild( cl, "match_depth",
-					 toString<double>( api->matchDepth()) );
+			TiCC::XmlNewTextChild( cl, "match_depth",
+					       TiCC::toString<double>( api->matchDepth()) );
 		      }
 		      if ( api->Verbosity(NEAR_N) ){
 			xmlNode *nb = api->bestNeighborsToXML();
@@ -791,7 +671,7 @@ void HttpServer::callback( childArgs *args ){
 	      }
 	    }
 	    else {
-	      *Dbg(myLog) << "invalid BASE! '" << basename
+	      DBG << "invalid BASE! '" << basename
 			  << "'" << endl;
 	      args->os() << "invalid basename: '" << basename << "'" << endl;
 	    }
@@ -803,93 +683,66 @@ void HttpServer::callback( childArgs *args ){
   }
 }
 
-
-ServerBase *startServer( TimblOpts& opts ){
-  bool mood;
-  string value;
-  Configuration *config = new Configuration();
-  bool old = false;
-  if ( !opts.Find( "config", value, mood ) ){
-    if ( opts.Find( 'S', value, mood ) ){
-      config->setatt( "port", value );
-      old = true;
-      if ( opts.Find( 'C', value, mood ) ){
-	config->setatt( "maxconn", value );
-      }
-    }
-    if ( !old ){
-      cerr << "missing --config option" << endl;
-      return 0;
-    }
-  }
-  else if ( !config->fill( value ) ){
-    cerr << "unable to read a configuration from " << value << endl;
-    return 0;
-  }
-  if ( opts.Find( "pidfile", value, mood ) ){
-    config->setatt( "pidfile", value );
-  }
-  if ( opts.Find( "logfile", value, mood ) ){
-    config->setatt( "logfile", value );
-  }
-  if ( opts.Find( "daemonize", value, mood ) ){
-    config->setatt( "daemonize", value );
-  }
-  if ( opts.Find( "debug", value, mood ) ){
-    config->setatt( "debug", value );
-    opts.Delete( "debug" );
-  }
-  string protocol = config->lookUp( "protocol" );
-  if ( protocol.empty() )
-    protocol = "tcp";
-  if ( protocol == "tcp" )
-    return new TcpServer( config );
-  else if ( protocol == "http" )
-    return new HttpServer( config );
-  else {
-    cerr << "unknown protocol " << protocol << endl;
-    return 0;
-  }
-}
-
 int main(int argc, char *argv[]){
   try {
-    struct tm *curtime;
-    time_t Time;
     // Start.
     //
-    cerr << "TiMBL Server " << TimblServer::Version() << "-NT"
-	 << " (c) ILK 1998 - 2014.\n"
+    cerr << "TiMBL Server " << TimblServer::Version()
+	 << " (c) CLTS/ILK/CLIPS 1998 - 2016.\n"
 	 << "Tilburg Memory Based Learner\n"
+	 << "Centre for Language and Speech Technology, Radboud University\n"
 	 << "Induction of Linguistic Knowledge Research Group, Tilburg University\n"
 	 << "CLiPS Computational Linguistics Group, University of Antwerp\n"
 	 << "based on " << Timbl::VersionName() << endl;
-    time(&Time);
-    curtime = localtime(&Time);
-    cerr << asctime(curtime) << endl;
+    cerr << TiCC::Timer::now();
     if ( argc <= 1 ){
       usage();
       return 1;
     }
-    TimblOpts Opts( argc, argv );
-    Preset_Values( Opts );
-    ServerBase *server = startServer( Opts );
-    if ( Do_Server ){
-      // Special case:   running a classic Server
-      startClassicExperiment( server, Opts );
+
+    TiCC::CL_Options opts;
+    opts.set_short_options( timbl_short_opts + serv_short_opts );
+    opts.set_long_options( timbl_long_opts + serv_long_opts );
+    opts.init( argc, argv );
+    string value;
+    if ( opts.is_present( 'h' )
+	 || opts.is_present( "help" ) ){
+      usage_full();
+      exit(EXIT_SUCCESS);
     }
-    else if ( Do_Multi_Server ){
-      startExperiments( server, Opts );
+    if ( opts.is_present( 'V' )
+	 || opts.is_present( "version" ) ){
+      cerr << "TiMBL server " << ServerBase::VersionInfo( true ) << endl;
+      cerr << "Based on TiMBL " << TimblAPI::VersionInfo( true ) << endl;
+      exit(EXIT_SUCCESS);
     }
+
+    opts.insert( 'v', "F", true );
+    opts.insert( 'v', "S", false );
+    TiCC::Configuration *config = initServerConfig( opts );
+    if ( !config ){
+      exit(EXIT_FAILURE);
+    }
+    ServerBase *server = 0;
+    string protocol = config->lookUp( "protocol" );
+    if ( protocol.empty() )
+      protocol = "tcp";
+    if ( protocol == "tcp" )
+      server = new TcpServer( config );
+    else if ( protocol == "http" )
+      server = new HttpServer( config );
+    else {
+      cerr << "unknown protocol " << protocol << endl;
+      exit(EXIT_FAILURE);
+    }
+    server->myLog.message("timblserver");
+    startExperiments( server );
     return server->Run(); // returns EXIT_SUCCESS or EXIT_FAIL
   }
   catch(std::bad_alloc){
     cerr << "ran out of memory somewhere" << endl;
     cerr << "timblserver terminated, Sorry for that" << endl;
   }
-  catch( softExit& e ){
-    return 0;
-  }
   catch(std::string& what){
     cerr << "an exception was raised: '" << what << "'" << endl;
     cerr << "timblserver terminated, Sorry for that" << endl;
